(*********************************************************************************************)
(******                FeynRules mod-file for Dark Matter contact interactions          ******)
(******                                                                                 ******)
(******                                    Abbreviations                                ******)
(******                                CS = complex scalar DM                           ******)
(******                                RS = real scalar DM                              ******)
(******                                DF = Dirac fermion DM                            ******)
(******                                MF = Majorana fermion DM                         ******)
(******                                CV = complex vector DM                           ******)
(******                                RV = real vector DM                              ******)
(******                                                                                 ******)
(******     		Authors: A. Belyaev, L. Panizzi, A. Pukhov, M. Thomas   	******)
(******                                                                                 ******)
(*********************************************************************************************)
(* Contact interaction lagrangian for Maorana fermions not supported in MG, so everything related to it is commented*)

M$ModelName = "DM_ContactInteractions";

M$Information = {Authors -> {"A. Belyaev", "L.Panizzi", "A. Pukhov", "M. Thomas"}, 
              Version -> "1.0",
              Date -> "01/12/2015", 
              Institutions -> {"University of Southampton (School of Physics and Astronomy)", "University of Southampton (School of Physics and Astronomy)", "Moscow State University (Skobeltsyn Institute of Nuclear Physics)", "University of Southampton (School of Physics and Astronomy)"},
              Emails -> {"a.belyaev@soton.ac.uk", "l.panizzi@soton.ac.uk", "pukhov@lapp.in2p3.fr", "m.c.thomas@soton.ac.uk"}};

              
              
FeynmanGauge = True;

M$InteractionOrderHierarchy = {
 {QCD, 1}, {QED, 2},
 {CRS1 , 2}, {CRS2, 2}, {CRS5 , 2}, {CRS6, 2},
 {CCS1 , 2}, {CCS2, 2}, {CCS3 , 2}, {CCS4, 2}, {CCS5 , 2}, {CCS6, 2},
 {CDF1 , 2}, {CDF2 , 2}, {CDF3 , 2}, {CDF4 , 2}, {CDF1T, 2}, {CDF2T, 2}, {CDF3T, 2}, {CDF4T, 2}, {CDF5 , 2}, {CDF6 , 2}, {CDF7 , 2}, {CDF8 , 2}, {CDF9 , 2}, {CDF10, 2},
 (*{CMF1 , 2}, {CMF2 , 2}, {CMF3 , 2}, {CMF4 , 2}, {CMF1T, 2}, {CMF2T, 2}, {CMF3T, 2}, {CMF4T, 2}, {CMF9 , 2}, {CMF10, 2},*)
 {CRV1 , 2}, {CRV2 , 2}, {CRV11 , 2}, {CRV11A , 2},
 {CCV1 , 2}, {CCV2 , 2}, {CCV3 , 2}, {CCV4 , 2}, {CCV5 , 2}, {CCV6 , 2}, {CCV7M , 2}, {CCV7P , 2}, {CCV8M , 2}, {CCV8P , 2}, {CCV9M , 2}, {CCV9P , 2}, {CCV10M , 2}, {CCV10P , 2}, {CCV11 , 2}, {CCV11A , 2},
 (* operators with mq coefficient *)
 {CRS1Q , 2}, {CRS2Q, 2}, 
 {CCS1Q , 2}, {CCS2Q, 2},
 {CDF1Q , 2}, {CDF2Q , 2}, {CDF3Q , 2}, {CDF4Q , 2},
 (*{CMF1Q , 2}, {CMF2Q , 2}, {CMF3Q , 2}, {CMF4Q , 2},*)
 {CRV1Q , 2}, {CRV2Q , 2},
 {CCV1Q , 2}, {CCV2Q , 2}, {CCV5Q , 2}, {CCV6Q , 2} 
};



(* ************************** *)
(* *****    Particles   ***** *)
(* ************************** *)

(* warning, the second and third figure of the PDG code must be different if particles have antiparticles (otherwise lutran.f in pythia 6 reverts the pdg code to 0) *)

M$ClassesDescription = {

(* Real Scalar DM *)

  S[111] == { 
              ClassName -> rsdm,  
	      ParticleName -> "rsdm", 
	      SelfConjugate -> True, 
	      Mass -> {Mrsdm, 70}, 
	      Width -> {Wrsdm, 0}, 
	      PropagatorLabel -> {"rsdm"}, 
	      PropagatorType -> D, 
	      PropagatorArrow -> None, 
	      PDG -> {6000601}, 
	      FullName -> {"rsdm"}
	    },

(* Complex Scalar DM *)

  S[112] == { 
              ClassName -> csdm,  
	      ParticleName -> "csdm", 
	      SelfConjugate -> False, 
	      Mass -> {Mcsdm, 70}, 
	      Width -> {Wcsdm, 0}, 
	      PropagatorLabel -> {"csdm"}, 
	      PropagatorType -> D, 
	      PropagatorArrow -> Forward, 
	      PDG -> {6000602}, 
	      FullName -> {"csdm"}
	    },
 
(* Majorana Fermion Dark Matter *) 

(*  
  F[111] == { 
              ClassName -> mfdm, 
	      ParticleName -> "mfdm", 
	      SelfConjugate -> True, 
	      Mass -> {Mmfdm, 70}, 
	      Width -> {Wmfdm, 0}, 
	      PropagatorLabel -> {"mfdm"}, 
	      PropagatorType -> Straight, 
	      PropagatorArrow -> None, 
	      PDG -> {6000611}, 
	      FullName -> {"mfdm"}
	    },
*)


(* Dirac Fermion Dark Matter *) 

  F[112] == { 
              ClassName -> dfdm,  
	      ParticleName -> "dfdm", 
	      SelfConjugate -> False, 
	      Mass -> {Mdfdm, 70}, 
	      Width -> {Wdfdm, 0}, 
	      PropagatorLabel -> {"dfdm"}, 
	      PropagatorType -> Straight, 
	      PropagatorArrow -> Forward, 
	      PDG -> {6000612}, 
	      FullName -> {"dfdm"}
	    },

(* Real Vector Dark Matter *) 

  V[111] == { 
              ClassName -> rvdm, 
	      ParticleName -> "rvdm", 
	      SelfConjugate -> True, 
	      Mass -> {Mrvdm, 70}, 
	      Width -> {Wrvdm, 0}, 
	      PropagatorLabel -> {"rvdm"}, 
	      PropagatorType -> W, 
	      PropagatorArrow -> None, 
	      PDG -> {6000621}, 
	      FullName -> {"rvdm"}
	    },

(* Complex Vector Dark Matter *) 

  V[112] == { 
              ClassName -> cvdm, 
	      ParticleName -> "cvdm", 
	      SelfConjugate -> False, 
	      Mass -> {Mcvdm, 70}, 
	      Width -> {Wcvdm, 0}, 
	      PropagatorLabel -> {"cvdm"}, 
	      PropagatorType -> W, 
	      PropagatorArrow -> None, 
	      PDG -> {6000622}, 
	      FullName -> {"cvdm"}
	    }


};

(* ************************** *)
(* *****   Parameters   ***** *)
(* ************************** *)

M$Parameters = {
    
  (* External parameters, Real Scalar DM *)

  CRS1  == { ParameterType -> External, BlockName -> REALSCALAR, Value -> 0, Description -> "CRS1  parameter", InteractionOrder -> {CRS1 , 1} },
  CRS2  == { ParameterType -> External, BlockName -> REALSCALAR, Value -> 0, Description -> "CRS2  parameter", InteractionOrder -> {CRS2 , 1} },
  CRS1Q == { ParameterType -> External, BlockName -> REALSCALAR, Value -> 0, Description -> "CRS1Q parameter", InteractionOrder -> {CRS1Q, 1} },
  CRS2Q == { ParameterType -> External, BlockName -> REALSCALAR, Value -> 0, Description -> "CRS2Q parameter", InteractionOrder -> {CRS2Q, 1} },
  CRS5  == { ParameterType -> External, BlockName -> REALSCALAR, Value -> 0, Description -> "CRS5  parameter", InteractionOrder -> {CRS5 , 1} },
  CRS6  == { ParameterType -> External, BlockName -> REALSCALAR, Value -> 0, Description -> "CRS6  parameter", InteractionOrder -> {CRS6 , 1} },

  (* External parameters, Complex Scalar DM *)

  CCS1  == { ParameterType -> External, BlockName -> COMPLEXSCALAR, Value -> 0, Description -> "CCS1  parameter", InteractionOrder -> {CCS1 , 1} },
  CCS2  == { ParameterType -> External, BlockName -> COMPLEXSCALAR, Value -> 0, Description -> "CCS2  parameter", InteractionOrder -> {CCS2 , 1} },
  CCS1Q == { ParameterType -> External, BlockName -> COMPLEXSCALAR, Value -> 0, Description -> "CCS1Q parameter", InteractionOrder -> {CCS1Q, 1} },
  CCS2Q == { ParameterType -> External, BlockName -> COMPLEXSCALAR, Value -> 0, Description -> "CCS2Q parameter", InteractionOrder -> {CCS2Q, 1} },
  CCS3  == { ParameterType -> External, BlockName -> COMPLEXSCALAR, Value -> 0, Description -> "CCS3  parameter", InteractionOrder -> {CCS3 , 1} },
  CCS4  == { ParameterType -> External, BlockName -> COMPLEXSCALAR, Value -> 0, Description -> "CCS4  parameter", InteractionOrder -> {CCS4 , 1} },
  CCS5  == { ParameterType -> External, BlockName -> COMPLEXSCALAR, Value -> 0, Description -> "CCS5  parameter", InteractionOrder -> {CCS5 , 1} },
  CCS6  == { ParameterType -> External, BlockName -> COMPLEXSCALAR, Value -> 0, Description -> "CCS6  parameter", InteractionOrder -> {CCS6 , 1} },

  (* External parameters, Weyl Fermion DM *)

  CDF1  == { ParameterType -> External, BlockName -> DIRACFERMION, Value -> 0, Description -> "CDF1  parameter", InteractionOrder -> {CDF1 , 1} },
  CDF2  == { ParameterType -> External, BlockName -> DIRACFERMION, Value -> 0, Description -> "CDF2  parameter", InteractionOrder -> {CDF2 , 1} },
  CDF3  == { ParameterType -> External, BlockName -> DIRACFERMION, Value -> 0, Description -> "CDF3  parameter", InteractionOrder -> {CDF3 , 1} },
  CDF4  == { ParameterType -> External, BlockName -> DIRACFERMION, Value -> 0, Description -> "CDF4  parameter", InteractionOrder -> {CDF4 , 1} },
  CDF1Q == { ParameterType -> External, BlockName -> DIRACFERMION, Value -> 0, Description -> "CDF1Q parameter", InteractionOrder -> {CDF1Q, 1} },
  CDF2Q == { ParameterType -> External, BlockName -> DIRACFERMION, Value -> 0, Description -> "CDF2Q parameter", InteractionOrder -> {CDF2Q, 1} },
  CDF3Q == { ParameterType -> External, BlockName -> DIRACFERMION, Value -> 0, Description -> "CDF3Q parameter", InteractionOrder -> {CDF3Q, 1} },
  CDF4Q == { ParameterType -> External, BlockName -> DIRACFERMION, Value -> 0, Description -> "CDF4Q parameter", InteractionOrder -> {CDF4Q, 1} },
  CDF1T == { ParameterType -> External, BlockName -> DIRACFERMION, Value -> 0, Description -> "CDF1T parameter", InteractionOrder -> {CDF1T, 1} },
  CDF2T == { ParameterType -> External, BlockName -> DIRACFERMION, Value -> 0, Description -> "CDF2T parameter", InteractionOrder -> {CDF2T, 1} },
  CDF3T == { ParameterType -> External, BlockName -> DIRACFERMION, Value -> 0, Description -> "CDF3T parameter", InteractionOrder -> {CDF3T, 1} },
  CDF4T == { ParameterType -> External, BlockName -> DIRACFERMION, Value -> 0, Description -> "CDF4T parameter", InteractionOrder -> {CDF4T, 1} },
  CDF5  == { ParameterType -> External, BlockName -> DIRACFERMION, Value -> 0, Description -> "CDF5  parameter", InteractionOrder -> {CDF5 , 1} },
  CDF6  == { ParameterType -> External, BlockName -> DIRACFERMION, Value -> 0, Description -> "CDF6  parameter", InteractionOrder -> {CDF6 , 1} },
  CDF7  == { ParameterType -> External, BlockName -> DIRACFERMION, Value -> 0, Description -> "CDF7  parameter", InteractionOrder -> {CDF7 , 1} },
  CDF8  == { ParameterType -> External, BlockName -> DIRACFERMION, Value -> 0, Description -> "CDF8  parameter", InteractionOrder -> {CDF8 , 1} },
  CDF9  == { ParameterType -> External, BlockName -> DIRACFERMION, Value -> 0, Description -> "CDF9  parameter", InteractionOrder -> {CDF9 , 1} },
  CDF10 == { ParameterType -> External, BlockName -> DIRACFERMION, Value -> 0, Description -> "CDF10 parameter", InteractionOrder -> {CDF10, 1} },

  (* External parameters, Majorana Fermion DM *)

(*
  CMF1  == { ParameterType -> External, BlockName -> MAJORANAFERMION, Value -> 0, Description -> "CMF1  parameter", InteractionOrder -> {CMF1 , 1} },
  CMF2  == { ParameterType -> External, BlockName -> MAJORANAFERMION, Value -> 0, Description -> "CMF2  parameter", InteractionOrder -> {CMF2 , 1} },
  CMF3  == { ParameterType -> External, BlockName -> MAJORANAFERMION, Value -> 0, Description -> "CMF3  parameter", InteractionOrder -> {CMF3 , 1} },
  CMF4  == { ParameterType -> External, BlockName -> MAJORANAFERMION, Value -> 0, Description -> "CMF4  parameter", InteractionOrder -> {CMF4 , 1} },
  CMF1Q == { ParameterType -> External, BlockName -> MAJORANAFERMION, Value -> 0, Description -> "CMF1Q parameter", InteractionOrder -> {CMF1Q, 1} },
  CMF2Q == { ParameterType -> External, BlockName -> MAJORANAFERMION, Value -> 0, Description -> "CMF2Q parameter", InteractionOrder -> {CMF2Q, 1} },
  CMF3Q == { ParameterType -> External, BlockName -> MAJORANAFERMION, Value -> 0, Description -> "CMF3Q parameter", InteractionOrder -> {CMF3Q, 1} },
  CMF4Q == { ParameterType -> External, BlockName -> MAJORANAFERMION, Value -> 0, Description -> "CMF4Q parameter", InteractionOrder -> {CMF4Q, 1} },
  CMF1T == { ParameterType -> External, BlockName -> MAJORANAFERMION, Value -> 0, Description -> "CMF1T parameter", InteractionOrder -> {CMF1T, 1} },
  CMF2T == { ParameterType -> External, BlockName -> MAJORANAFERMION, Value -> 0, Description -> "CMF2T parameter", InteractionOrder -> {CMF2T, 1} },
  CMF3T == { ParameterType -> External, BlockName -> MAJORANAFERMION, Value -> 0, Description -> "CMF3T parameter", InteractionOrder -> {CMF3T, 1} },
  CMF4T == { ParameterType -> External, BlockName -> MAJORANAFERMION, Value -> 0, Description -> "CMF4T parameter", InteractionOrder -> {CMF4T, 1} },
  CMF9  == { ParameterType -> External, BlockName -> MAJORANAFERMION, Value -> 0, Description -> "CMF9  parameter", InteractionOrder -> {CMF9 , 1} },
  CMF10 == { ParameterType -> External, BlockName -> MAJORANAFERMION, Value -> 0, Description -> "CMF10 parameter", InteractionOrder -> {CMF10, 1} },
*)

  (* External parameters, Real Vector DM *)

  CRV1    == { ParameterType -> External, BlockName -> REALVECTOR, Value -> 0, Description -> "CRV1   parameter", InteractionOrder -> {CRV1  , 1} },
  CRV2    == { ParameterType -> External, BlockName -> REALVECTOR, Value -> 0, Description -> "CRV2   parameter", InteractionOrder -> {CRV2  , 1} },
  CRV1Q   == { ParameterType -> External, BlockName -> REALVECTOR, Value -> 0, Description -> "CRV1Q  parameter", InteractionOrder -> {CRV1Q , 1} },
  CRV2Q   == { ParameterType -> External, BlockName -> REALVECTOR, Value -> 0, Description -> "CRV2Q  parameter", InteractionOrder -> {CRV2Q , 1} },
  CRV11   == { ParameterType -> External, BlockName -> REALVECTOR, Value -> 0, Description -> "CRV11  parameter", InteractionOrder -> {CRV11 , 1} },
  CRV11A  == { ParameterType -> External, BlockName -> REALVECTOR, Value -> 0, Description -> "CRV11A parameter", InteractionOrder -> {CRV11A, 1} },

  (* External parameters, Complex Vector DM *)

  CCV1    == { ParameterType -> External, BlockName -> COMPLEXVECTOR, Value -> 0, Description -> "CCV1   parameter", InteractionOrder -> {CCV1  , 1} },
  CCV2    == { ParameterType -> External, BlockName -> COMPLEXVECTOR, Value -> 0, Description -> "CCV2   parameter", InteractionOrder -> {CCV2  , 1} },
  CCV1Q   == { ParameterType -> External, BlockName -> COMPLEXVECTOR, Value -> 0, Description -> "CCV1Q  parameter", InteractionOrder -> {CCV1Q , 1} },
  CCV2Q   == { ParameterType -> External, BlockName -> COMPLEXVECTOR, Value -> 0, Description -> "CCV2Q  parameter", InteractionOrder -> {CCV2Q , 1} },
  CCV3    == { ParameterType -> External, BlockName -> COMPLEXVECTOR, Value -> 0, Description -> "CCV3   parameter", InteractionOrder -> {CCV3  , 1} },
  CCV4    == { ParameterType -> External, BlockName -> COMPLEXVECTOR, Value -> 0, Description -> "CCV4   parameter", InteractionOrder -> {CCV4  , 1} },
  CCV5    == { ParameterType -> External, BlockName -> COMPLEXVECTOR, Value -> 0, Description -> "CCV5   parameter", InteractionOrder -> {CCV5  , 1} },
  CCV6    == { ParameterType -> External, BlockName -> COMPLEXVECTOR, Value -> 0, Description -> "CCV6   parameter", InteractionOrder -> {CCV6  , 1} },
  CCV5Q   == { ParameterType -> External, BlockName -> COMPLEXVECTOR, Value -> 0, Description -> "CCV5Q  parameter", InteractionOrder -> {CCV5Q , 1} },
  CCV6Q   == { ParameterType -> External, BlockName -> COMPLEXVECTOR, Value -> 0, Description -> "CCV6Q  parameter", InteractionOrder -> {CCV6Q , 1} },
  CCV7M   == { ParameterType -> External, BlockName -> COMPLEXVECTOR, Value -> 0, Description -> "CCV7M  parameter", InteractionOrder -> {CCV7M , 1} },
  CCV7P   == { ParameterType -> External, BlockName -> COMPLEXVECTOR, Value -> 0, Description -> "CCV7P  parameter", InteractionOrder -> {CCV7P , 1} },
  CCV8M   == { ParameterType -> External, BlockName -> COMPLEXVECTOR, Value -> 0, Description -> "CCV8M  parameter", InteractionOrder -> {CCV8M , 1} },
  CCV8P   == { ParameterType -> External, BlockName -> COMPLEXVECTOR, Value -> 0, Description -> "CCV8P  parameter", InteractionOrder -> {CCV8P , 1} },
  CCV9M   == { ParameterType -> External, BlockName -> COMPLEXVECTOR, Value -> 0, Description -> "CCV9M  parameter", InteractionOrder -> {CCV9M , 1} },
  CCV9P   == { ParameterType -> External, BlockName -> COMPLEXVECTOR, Value -> 0, Description -> "CCV9P  parameter", InteractionOrder -> {CCV9P , 1} },
  CCV10M  == { ParameterType -> External, BlockName -> COMPLEXVECTOR, Value -> 0, Description -> "CCV10M parameter", InteractionOrder -> {CCV10M, 1} },
  CCV10P  == { ParameterType -> External, BlockName -> COMPLEXVECTOR, Value -> 0, Description -> "CCV10P parameter", InteractionOrder -> {CCV10P, 1} },
  CCV11   == { ParameterType -> External, BlockName -> COMPLEXVECTOR, Value -> 0, Description -> "CCV11  parameter", InteractionOrder -> {CCV11 , 1} },
  CCV11A  == { ParameterType -> External, BlockName -> COMPLEXVECTOR, Value -> 0, Description -> "CCV11A parameter", InteractionOrder -> {CCV11A, 1} },


  Lambda == { ParameterType -> External, BlockName -> LAMBDAUV, Value -> 10000. , Description -> "cutoff scale" }


};





(* ************************** *)
(* *****   Lagrangian   ***** *)
(* ************************** *)

(* DM Kinetic and Mass term Lagrangians *)

LRSDM := 1/2 DC[rsdm, mu] DC[rsdm,mu] - 1/2 Mrsdm^2 rsdm^2; 
LCSDM := 1/2 ( DC[csdmbar, mu] DC[csdm,mu] + HC[DC[csdmbar, mu] DC[csdm,mu]] ) - Mcsdm^2 csdmbar csdm; 
LDFDM := I dfdmbar.Ga[mu].DC[dfdm, mu]- Mdfdm dfdmbar.dfdm;
(*LMFDM := I/2 mfdmbar.Ga[mu].DC[mfdm,mu] - 1/2 Mmfdm mfdmbar.mfdm;*)
LRVDM := -1/4 (DC[rvdm[nu], mu] - DC[rvdm[mu],nu]) (DC[rvdm[nu], mu] - DC[rvdm[mu],nu]) + 1/2 Mrvdm^2 rvdm[mu] rvdm[mu];
LCVDM := -1/2 (DC[cvdmbar[nu], mu] - DC[cvdmbar[mu],nu]) (DC[cvdm[nu], mu] - DC[cvdm[mu],nu]) + HC[-1/2 (DC[cvdmbar[nu], mu] - DC[cvdmbar[mu],nu]) (DC[cvdm[nu], mu] - DC[cvdm[mu],nu]) ] + Mcvdm^2 cvdmbar[mu] cvdm[mu];

(* Real Scalar DM *)

LRS1  := 1/2 1/Lambda^2 CRS1  rsdm rsdm ( uqbar.uq + dqbar.dq ); 
LRS2  := 1/2 I/Lambda^2 CRS2  rsdm rsdm ( uqbar.Ga[5].uq + dqbar.Ga[5].dq ); 
LRS1Q := 1/2 1/Lambda^2 CRS1Q rsdm rsdm ( Mu[ff] uqbar[ss,ff,cc].uq[ss,ff,cc] + Md[ff] dqbar[ss,ff,cc].dq[ss,ff,cc] ); 
LRS2Q := 1/2 I/Lambda^2 CRS2Q rsdm rsdm ( Mu[ff] uqbar[s1,ff,cc].Ga[5,s1,s2].uq[s2,ff,cc] + Md[ff] dqbar[s1,ff,cc].Ga[5,s1,s2].dq[s2,ff,cc] ); 
LRS5  := 1/2 1/Lambda^2 CRS5  rsdm rsdm FS[G,mu,nu,a] FS[G,mu,nu,a]; 
LRS6  := 1/2 1/Lambda^2 CRS6  rsdm rsdm FS[G,mu,nu,a] 1/2 Eps[alpha,beta,mu,nu] FS[G,alpha,beta,a];

(* Complex Scalar DM *)

LCS1  := 1/Lambda^2 CCS1  csdmbar csdm ( uqbar.uq + dqbar.dq ); 
LCS2  := I/Lambda^2 CCS2  csdmbar csdm ( uqbar.Ga[5].uq + dqbar.Ga[5].dq ); 
LCS1Q := 1/Lambda^2 CCS1Q csdmbar csdm ( Mu[ff] uqbar[ss,ff,cc].uq[ss,ff,cc] + Md[ff] dqbar[ss,ff,cc].dq[ss,ff,cc] ); 
LCS2Q := I/Lambda^2 CCS2Q csdmbar csdm ( Mu[ff] uqbar[s1,ff,cc].Ga[5,s1,s2].uq[s2,ff,cc] + Md[ff] dqbar[s1,ff,cc].Ga[5,s1,s2].dq[s2,ff,cc] ); 
LCS3  := I/Lambda^2 CCS3  csdmbar csdm ( uqbar.Ga[mu].DC[uq, mu] - DC[uqbar,mu].Ga[mu].uq + dqbar.Ga[mu].DC[dq, mu] - DC[dqbar,mu].Ga[mu].dq ); 
LCS4  := I/Lambda^2 CCS4  csdmbar csdm ( uqbar.Ga[mu].Ga[5].DC[uq, mu] - DC[uqbar,mu].Ga[mu].Ga[5].uq + dqbar.Ga[mu].Ga[5].DC[dq,mu] - DC[dqbar,mu].Ga[mu].Ga[5].dq ); 
LCS5  := 1/Lambda^2 CCS5  csdmbar csdm FS[G,mu,nu,a] FS[G,mu,nu,a]; 
LCS6  := 1/Lambda^2 CCS6  csdmbar csdm FS[G,mu,nu,a] 1/2 Eps[alpha,beta,mu,nu] FS[G,alpha,beta,a]; 

(* Dirac Fermion DM *)

LDF1  := 1/Lambda^2 CDF1  dfdmbar.dfdm ( uqbar.uq + dqbar.dq ); 
LDF2  := I/Lambda^2 CDF2  dfdmbar.Ga[5].dfdm ( uqbar.uq + dqbar.dq ); 
LDF3  := I/Lambda^2 CDF3  dfdmbar.dfdm ( uqbar.Ga[5].uq + dqbar.Ga[5].dq ); 
LDF4  := 1/Lambda^2 CDF4  dfdmbar.Ga[5].dfdm ( uqbar.Ga[5].uq + dqbar.Ga[5].dq ); 
LDF1Q := 1/Lambda^3 CDF1Q dfdmbar.dfdm ( Mu[ff] uqbar[ss,ff,cc].uq[ss,ff,cc] + Md[ff] dqbar[ss,ff,cc].dq[ss,ff,cc] ); 
LDF2Q := I/Lambda^3 CDF2Q dfdmbar.Ga[5].dfdm ( Mu[ff] uqbar[ss,ff,cc].uq[ss,ff,cc] + Md[ff] dqbar[ss,ff,cc].dq[ss,ff,cc] ); 
LDF3Q := I/Lambda^3 CDF3Q dfdmbar.dfdm ( Mu[ff] uqbar[s1,ff,cc].Ga[5,s1,s2].uq[s2,ff,cc] + Md[ff] dqbar[s1,ff,cc].Ga[5,s1,s2].dq[s2,ff,cc] ); 
LDF4Q := 1/Lambda^3 CDF4Q dfdmbar.Ga[5].dfdm ( Mu[ff] uqbar[s1,ff,cc].Ga[5,s1,s2].uq[s2,ff,cc] + Md[ff] dqbar[s1,ff,cc].Ga[5,s1,s2].dq[s2,ff,cc] ); 
LDF1T := 1/Lambda^2 CDF1T ( dfdmbar[s1].uq[s1,ff1,cc] uqbar[s2,ff1,cc].dfdm[s2] + dfdmbar[s1].dq[s1,ff1,cc] dqbar[s2,ff1,cc].dfdm[s2]); 
LDF2T := I/(2*Lambda^2) CDF2T ( dfdmbar[s1].Ga[5,s1,s2].uq[s2,ff1,cc] uqbar[s3,ff1,cc].dfdm[s3] + dfdmbar[s1].uq[s1,ff1,cc] uqbar[s2,ff1,cc].Ga[5,s2,s3].dfdm[s3]
                              + dfdmbar[s1].Ga[5,s1,s2].dq[s2,ff1,cc] dqbar[s3,ff1,cc].dfdm[s3] + dfdmbar[s1].dq[s1,ff1,cc] dqbar[s2,ff1,cc].Ga[5,s2,s3].dfdm[s3] ); 
LDF3T := 1/(2*Lambda^2) CDF3T ( dfdmbar[s1].Ga[5,s1,s2].uq[s2,ff1,cc] uqbar[s3,ff1,cc].dfdm[s3] - dfdmbar[s1].uq[s1,ff1,cc] uqbar[s2,ff1,cc].Ga[5,s2,s3].dfdm[s3]
                              + dfdmbar[s1].Ga[5,s1,s2].dq[s2,ff1,cc] dqbar[s3,ff1,cc].dfdm[s3] - dfdmbar[s1].dq[s1,ff1,cc] dqbar[s2,ff1,cc].Ga[5,s2,s3].dfdm[s3] ); 
LDF4T := 1/Lambda^2 CDF4T ( dfdmbar[s1].Ga[5,s1,s2].uq[s2,ff1,cc] uqbar[s3,ff1,cc].Ga[5,s3,s4].dfdm[s4] + dfdmbar[s1].Ga[5,s1,s2].dq[s2,ff1,cc] dqbar[s3,ff1,cc].Ga[5,s3,s4].dfdm[s4] );
LDF5  := 1/Lambda^2 CDF5  dfdmbar.Ga[mu].dfdm ( uqbar.Ga[mu].uq + dqbar.Ga[mu].dq ); 
LDF6  := 1/Lambda^2 CDF6  dfdmbar.Ga[mu].Ga[5].dfdm ( uqbar.Ga[mu].uq + dqbar.Ga[mu].dq ); 
LDF7  := 1/Lambda^2 CDF7  dfdmbar.Ga[mu].dfdm ( uqbar.Ga[mu].Ga[5].uq + dqbar.Ga[mu].Ga[5].dq ); 
LDF8  := 1/Lambda^2 CDF8  dfdmbar.Ga[mu].Ga[5].dfdm ( uqbar.Ga[mu].Ga[5].uq + dqbar.Ga[mu].Ga[5].dq ); 
LDF9  := 1/Lambda^2 CDF9  ( I/2 (dfdmbar.Ga[mu].Ga[nu].dfdm - dfdmbar.Ga[nu].Ga[mu].dfdm) 
                           I/2 ( uqbar.Ga[mu].Ga[nu].uq - uqbar.Ga[nu].Ga[mu].uq + dqbar.Ga[mu].Ga[nu].dq - dqbar.Ga[nu].Ga[mu].dq ) 
                          ); 
LDF10 := I/Lambda^2 CDF10 ( I/2 (dfdmbar.Ga[mu].Ga[nu].Ga[5].dfdm - dfdmbar.Ga[nu].Ga[mu].Ga[5].dfdm) 
                            I/2 ( uqbar.Ga[mu].Ga[nu].uq - uqbar.Ga[nu].Ga[mu].uq + dqbar.Ga[mu].Ga[nu].dq - dqbar.Ga[nu].Ga[mu].dq )
                          );



(* Majorana Fermion DM *)
(*
LMF1  := 1/2 1/Lambda^2 CMF1  mfdmbar.mfdm ( uqbar.uq + dqbar.dq ); 
LMF2  := 1/2 1/Lambda^2 CMF2  mfdmbar.Ga[5].mfdm ( uqbar.uq + dqbar.dq ); 
LMF3  := 1/2 1/Lambda^2 CMF3  mfdmbar.mfdm ( uqbar.Ga[5].uq + dqbar.Ga[5].dq ); 
LMF4  := 1/2 1/Lambda^2 CMF4  mfdmbar.Ga[5].mfdm ( uqbar.Ga[5].uq + dqbar.Ga[5].dq ); 
LMF1Q := 1/2 1/Lambda^3 CMF1Q mfdmbar.mfdm ( Mu[ff] uqbar[ss,ff,cc].uq[ss,ff,cc] + Md[ff] dqbar[ss,ff,cc].dq[ss,ff,cc] ); 
LMF2Q := 1/2 1/Lambda^3 CMF2Q mfdmbar.Ga[5].mfdm ( Mu[ff] uqbar[ss,ff,cc].uq[ss,ff,cc] + Md[ff] dqbar[ss,ff,cc].dq[ss,ff,cc] ); 
LMF3Q := 1/2 1/Lambda^3 CMF3Q mfdmbar.mfdm ( Mu[ff] uqbar[s1,ff,cc].Ga[5,s1,s2].uq[s2,ff,cc] + Md[ff] dqbar[s1,ff,cc].Ga[5,s1,s2].dq[s2,ff,cc] ); 
LMF4Q := 1/2 1/Lambda^3 CMF4Q mfdmbar.Ga[5].mfdm ( Mu[ff] uqbar[s1,ff,cc].Ga[5,s1,s2].uq[s2,ff,cc] + Md[ff] dqbar[s1,ff,cc].Ga[5,s1,s2].dq[s2,ff,cc] ); 
LMF1T := 1/2 1/Lambda^2 CMF1T ( mfdmbar[s1].uq[s1,ff1,cc] uqbar[s2,ff1,cc].mfdm[s2] + mfdmbar[s1].dq[s1,ff1,cc] dqbar[s2,ff1,cc].mfdm[s2]); 
LMF2T := 1/2 I/(2*Lambda^2) CMF2T ( mfdmbar[s1].Ga[5,s1,s2].uq[s2,ff1,cc] uqbar[s3,ff1,cc].mfdm[s3] + mfdmbar[s1].uq[s1,ff1,cc] uqbar[s2,ff1,cc].Ga[5,s2,s3].mfdm[s3]
                                  + mfdmbar[s1].Ga[5,s1,s2].dq[s2,ff1,cc] dqbar[s3,ff1,cc].mfdm[s3] + mfdmbar[s1].dq[s1,ff1,cc] dqbar[s2,ff1,cc].Ga[5,s2,s3].mfdm[s3] ); 
LMF3T := 1/2 1/(2*Lambda^2) CMF3T ( mfdmbar[s1].Ga[5,s1,s2].uq[s2,ff1,cc] uqbar[s3,ff1,cc].mfdm[s3] - mfdmbar[s1].uq[s1,ff1,cc] uqbar[s2,ff1,cc].Ga[5,s2,s3].mfdm[s3]
                                  + mfdmbar[s1].Ga[5,s1,s2].dq[s2,ff1,cc] dqbar[s3,ff1,cc].mfdm[s3] - mfdmbar[s1].dq[s1,ff1,cc] dqbar[s2,ff1,cc].Ga[5,s2,s3].mfdm[s3] ); 
LMF4T := 1/2 1/Lambda^2 CMF4T ( mfdmbar[s1].Ga[5,s1,s2].uq[s2,ff1,cc] uqbar[s3,ff1,cc].Ga[5,s3,s4].mfdm[s4] + mfdmbar[s1].Ga[5,s1,s2].dq[s2,ff1,cc] dqbar[s3,ff1,cc].Ga[5,s3,s4].mfdm[s4] );
LMF9  := 1/2 1/Lambda^2 CMF9  ( I/2 (mfdmbar.Ga[mu].Ga[nu].mfdm - mfdmbar.Ga[nu].Ga[mu].mfdm) 
                               I/2 ( uqbar.Ga[mu].Ga[nu].uq - uqbar.Ga[nu].Ga[mu].uq + dqbar.Ga[mu].Ga[nu].dq - dqbar.Ga[nu].Ga[mu].dq ) 
                              ); 
LMF10 := 1/2 I/Lambda^2 CMF10 ( I/2 (mfdmbar.Ga[mu].Ga[nu].Ga[5].mfdm - mfdmbar.Ga[nu].Ga[mu].Ga[5].mfdm) 
                                I/2 ( uqbar.Ga[mu].Ga[nu].uq - uqbar.Ga[nu].Ga[mu].uq + dqbar.Ga[mu].Ga[nu].dq - dqbar.Ga[nu].Ga[mu].dq )
                              );
*)
(* Real Vector DM *)

LRV1   := 1/2 1/Lambda^2 CRV1   rvdm[mu] rvdm[mu] ( uqbar.uq + dqbar.dq );
LRV2   := 1/2 I/Lambda^2 CRV2   rvdm[mu] rvdm[mu] ( uqbar.Ga[5].uq + dqbar.Ga[5].dq );
LRV1Q  := 1/2 1/Lambda^2 CRV1Q  rvdm[mu] rvdm[mu] ( Mu[ff] uqbar[ss,ff,cc].uq[ss,ff,cc] + Md[ff] dqbar[ss,ff,cc].dq[ss,ff,cc] ); 
LRV2Q  := 1/2 I/Lambda^2 CRV2Q  rvdm[mu] rvdm[mu] ( Mu[ff] uqbar[s1,ff,cc].Ga[5,s1,s2].uq[s2,ff,cc] + Md[ff] dqbar[s1,ff,cc].Ga[5,s1,s2].dq[s2,ff,cc] ); 
LRV11  := 1/2 1/Lambda^2 CRV11  rvdm[mu] rvdm[mu] FS[G,alpha,beta,a] FS[G,alpha,beta,a]; 
LRV11A := 1/2 1/Lambda^2 CRV11A rvdm[mu] rvdm[mu] FS[G,alpha,beta,a] 1/2 Eps[rho,sigma,alpha,beta] FS[G,rho,sigma,a];

(* Complex Vector DM (notation from 1508.04466) *)

LCV1   := 1/Lambda^2 CCV1   cvdmbar[mu] cvdm[mu] ( uqbar.uq + dqbar.dq );
LCV2   := I/Lambda^2 CCV2   cvdmbar[mu] cvdm[mu] ( uqbar.Ga[5].uq + dqbar.Ga[5].dq );
LCV1Q  := 1/Lambda^2 CCV1Q  cvdmbar[mu] cvdm[mu] ( Mu[ff] uqbar[ss,ff,cc].uq[ss,ff,cc] + Md[ff] dqbar[ss,ff,cc].dq[ss,ff,cc] ); 
LCV2Q  := I/Lambda^2 CCV2Q  cvdmbar[mu] cvdm[mu] ( Mu[ff] uqbar[s1,ff,cc].Ga[5,s1,s2].uq[s2,ff,cc] + Md[ff] dqbar[s1,ff,cc].Ga[5,s1,s2].dq[s2,ff,cc] ); 
LCV3   := 1/2 I/Lambda^2 CCV3   (cvdmbar[nu] del[cvdm[nu],mu] - cvdm[nu] del[cvdmbar[nu],mu]) ( uqbar.Ga[mu].uq + dqbar.Ga[mu].dq );
LCV4   := 1/2 I/Lambda^2 CCV4   (cvdmbar[nu] del[cvdm[nu],mu] - cvdm[nu] del[cvdmbar[nu],mu]) ( uqbar.Ga[mu].Ga[5].uq + dqbar.Ga[mu].Ga[5].dq );
LCV5   := I/Lambda^2 CCV5   cvdmbar[mu] cvdm[nu] (I/2 ( uqbar.Ga[mu].Ga[nu].uq - uqbar.Ga[nu].Ga[mu].uq + dqbar.Ga[mu].Ga[nu].dq - dqbar.Ga[nu].Ga[mu].dq ));
LCV6   := 1/Lambda^2 CCV6   cvdmbar[mu] cvdm[nu] (I/2 ( uqbar.Ga[mu].Ga[nu].Ga[5].uq - uqbar.Ga[nu].Ga[mu].Ga[5].uq + dqbar.Ga[mu].Ga[nu].Ga[5].dq - dqbar.Ga[nu].Ga[mu].Ga[5].dq ));
LCV5Q  := I/Lambda^2 CCV5Q  cvdmbar[mu] cvdm[nu] (I/2 ( Mu[ff] uqbar[s1,ff,cc].Ga[mu,s1,s2].Ga[nu,s2,s3].uq[s3,ff,cc] - Mu[ff] uqbar[s1,ff,cc].Ga[nu,s1,s2].Ga[mu,s2,s3].uq[s3,ff,cc] + Md[ff] dqbar[s1,ff,cc].Ga[mu,s1,s2].Ga[nu,s2,s3].dq[s3,ff,cc] - Md[ff] dqbar[s1,ff,cc].Ga[nu,s1,s2].Ga[mu,s2,s3].dq[s3,ff,cc] ));
LCV6Q  := 1/Lambda^2 CCV6Q  cvdmbar[mu] cvdm[nu] (I/2 ( Mu[ff] uqbar[s1,ff,cc].Ga[mu,s1,s2].Ga[nu,s2,s3].Ga[5,s3,s4].uq[s4,ff,cc] - Mu[ff] uqbar[s1,ff,cc].Ga[nu,s1,s2].Ga[mu,s2,s3].Ga[5,s3,s4].uq[s4,ff,cc] + Md[ff] dqbar[s1,ff,cc].Ga[mu,s1,s2].Ga[nu,s2,s3].Ga[5,s3,s4].dq[s4,ff,cc] - Md[ff] dqbar[s1,ff,cc].Ga[nu,s1,s2].Ga[mu,s2,s3].Ga[5,s3,s4].dq[s4,ff,cc] ));
LCV7P  := 1/2 1/Lambda^2 CCV7P  (cvdmbar[nu] del[cvdm[mu],nu] + cvdm[nu] del[cvdmbar[mu],nu]) ( uqbar.Ga[mu].uq + dqbar.Ga[mu].dq );
LCV7M  := 1/2 I/Lambda^2 CCV7M  (cvdmbar[nu] del[cvdm[mu],nu] - cvdm[nu] del[cvdmbar[mu],nu]) ( uqbar.Ga[mu].uq + dqbar.Ga[mu].dq );
LCV8P  := 1/2 1/Lambda^2 CCV8P  (cvdmbar[nu] del[cvdm[mu],nu] + cvdm[nu] del[cvdmbar[mu],nu]) ( uqbar.Ga[mu].Ga[5].uq + dqbar.Ga[mu].Ga[5].dq );
LCV8M  := 1/2 I/Lambda^2 CCV8M  (cvdmbar[nu] del[cvdm[mu],nu] - cvdm[nu] del[cvdmbar[mu],nu]) ( uqbar.Ga[mu].Ga[5].uq + dqbar.Ga[mu].Ga[5].dq );
LCV9P  := 1/2 1/Lambda^2 CCV9P  Eps[mu,nu,rho,sigma] (cvdmbar[nu] del[cvdm[sigma],rho] + cvdm[nu] del[cvdmbar[sigma],rho]) ( uqbar.Ga[mu].uq + dqbar.Ga[mu].dq );
LCV9M  := 1/2 I/Lambda^2 CCV9M  Eps[mu,nu,rho,sigma] (cvdmbar[nu] del[cvdm[sigma],rho] - cvdm[nu] del[cvdmbar[sigma],rho]) ( uqbar.Ga[mu].uq + dqbar.Ga[mu].dq );
LCV10P := 1/2 1/Lambda^2 CCV10P Eps[mu,nu,rho,sigma] (cvdmbar[nu] del[cvdm[sigma],rho] + cvdm[nu] del[cvdmbar[sigma],rho]) ( uqbar.Ga[mu].Ga[5].uq + dqbar.Ga[mu].Ga[5].dq );
LCV10M := 1/2 I/Lambda^2 CCV10M Eps[mu,nu,rho,sigma] (cvdmbar[nu] del[cvdm[sigma],rho] - cvdm[nu] del[cvdmbar[sigma],rho]) ( uqbar.Ga[mu].Ga[5].uq + dqbar.Ga[mu].Ga[5].dq );
LCV11  := 1/Lambda^2 CCV11  cvdmbar[mu] cvdm[mu] FS[G,alpha,beta,a] FS[G,alpha,beta,a]; 
LCV11A := 1/Lambda^2 CCV11A cvdmbar[mu] cvdm[mu] FS[G,alpha,beta,a] 1/2 Eps[rho,sigma,alpha,beta] FS[G,rho,sigma,a];

(*********************************)
(******** Total Lagrangians ******)
(*********************************)

(* Scalar DM *)
LRS := LRSDM + LRS1 + LRS2 + LRS1Q + LRS2Q + LRS5 + LRS6;
LCS := LCSDM + LCS1 + LCS2 + LCS1Q + LCS2Q + LCS3 + LCS4 + LCS5 + LCS6;

(* Fermion DM *)
LMF := LMFDM + LMF1 + LMF2 + LMF3 + LMF4 + LMF1Q + LMF2Q + LMF3Q + LMF4Q + LMF1T + LMF2T + LMF3T + LMF4T + LMF9 + LMF10;
LDF := LDFDM + LDF1 + LDF2 + LDF3 + LDF4 + LDF1Q + LDF2Q + LDF3Q + LDF4Q + LDF1T + LDF2T + LDF3T + LDF4T + LDF5 + LDF6 + LDF7 + LDF8 + LDF9 + LDF10;

(* Vector DM *)
LRV := LRVDM + LRV1 + LRV2 + LRV1Q + LRV2Q + LRV11 + LRV11A;
LCV := LCVDM + LCV1 + LCV2 + LCV1Q + LCV2Q + LCV3 + LCV4 + LCV5 + LCV6 + LCV5Q + LCV6Q + LCV7P + LCV7M + LCV8P + LCV8M + LCV9P + LCV9M + LCV10P + LCV10M + LCV11 + LCV11A;

(* Total *)
LCI := LRS + LCS + (*LMF +*) LDF + LRV + LCV;
